/*==============================================================================
LocoChat.

Plays Chat Audio Clips assigned by INI when players say specified words in game

Written by [ACE]CPan
==============================================================================*/
class MutChat expands Mutator config(LocoChat);

var bool bInitialized;

var SoundMedia SM;
var float PlayNext;
var int CurrentID;

var config string	ChatWords[255], StaticWords[25];		//All the words to check for
var config string	ChatSounds[255], StaticSounds[25];		//All the config sounds
var config float	ChatDuration[255], StaticDuration[25];	//config sounds length
var config bool	bAllowStatic,						//Allow to search for these words inside phrases
				bShowCommands,                          //Allow showing the commands on the HUD
				bUseASC;							//Set to True if you use ASC

var sound	sndChat[255], sndSChat[25];				//All sounds  ==  Maybe make dedicated ones for GG, Nice Shot, etc?

struct PlayerInfo
{
	var 	Pawn Player;
	var string PlayerName;
	var int 	PlayerID;
	var string NetworkAddress;
	var	SoundMedia SM;
};
var PlayerInfo PInfo[64];

function LoadSounds()
{
	local int		i;
	local sound	mysound;

	for (i=0; i<ArrayCount(ChatSounds); i++)
	{
		if ( ChatSounds[i] != "" )
		{
			mysound=sound(DynamicLoadObject(ChatSounds[i],class'Sound',false));
			if(mysound==none)
				Log("## Error Loading ==>> "$ChatSounds[i]);
			else
				sndChat[i] = mysound;
		}
	}
	for (i=0; i<ArrayCount(StaticSounds); i++)
	{
		if ( StaticSounds[i] != "" )
		{
			mysound=sound(DynamicLoadObject(StaticSounds[i],class'Sound',false));
			if(mysound==none)
				Log("## Error Loading ==>> "$StaticSounds[i]);
			else
				sndSChat[i] = mysound;
		}
	}
}
event PreBeginPlay()
{
	if ( bUseASC )
		CurrentID = 1;
	else
		CurrentID = 0;

	LoadSounds();

	if ( bShowCommands )
		spawn(class'Loco_HUD');
}

function PostBeginPlay()
{
	if (bInitialized)
		return;
	bInitialized = True;

	Level.Game.RegisterMessageMutator( self );
}

function ModifyPlayer(Pawn Other)
{
	if ( Level.Game.CurrentID > CurrentID )
		AddPlayers();

	super.ModifyPlayer(Other);
}

function tick(float DeltaTime)
{
	Super.tick(DeltaTime);
	if ( Level.Game.CurrentID > CurrentID )
		AddPlayers();
}

simulated function AddPlayers()
{
	local int i;
	local Pawn Other;
	local String TempAddress;
	local bool bfound;

	if ( Level.Game.CurrentID > CurrentID )
	{
		for( Other=Level.PawnList; Other!=None; Other=Other.NextPawn )
			if( Other.PlayerReplicationInfo.PlayerID == CurrentID)
				break;

		CurrentID++;

		if ( Other.IsA('Bot') )
    			return;

		TempAddress = FilterAddress(Other);

		for(i=0; i<ArrayCount(PInfo) && PInfo[i].PlayerName != ""; i++)
		{
			//	CHECK FOR IP ADDRESS
			if ( TempAddress ~= PInfo[i].NetworkAddress )
			{
				SetPInfo(Other, i);
				bFound = true;
			}
		}
		if ( !bFound )
		{
			for(i=0; i<ArrayCount(PInfo); i++)
			{
				if ( PlayerPawn(PInfo[i].Player) == none || PlayerPawn(PInfo[i].Player).Player == none )
				{
					PInfo[i].Player = none;
					PInfo[i].PlayerID = -1;
					PInfo[i].SM = none;
					PInfo[i].NetworkAddress = "";

					PInfo[i].Player = Other;
					PInfo[i].PlayerName = Other.PlayerReplicationInfo.PlayerName;
					PInfo[i].PlayerID = Other.PlayerReplicationInfo.PlayerID;
					PInfo[i].NetworkAddress = TempAddress;
					PInfo[i].SM = spawn(class'SoundMedia',Other,,Other.Location);
					PInfo[i].SM.ClientGiveDebug();
					PInfo[i].SM.ClientGiveMapM();
					PInfo[i].SM.ClientGivePerma();
					break;
				}
			}
		}
	}
}

function SetPInfo( Pawn Other, int Number )
{
	PInfo[Number].Player = Other;
	PInfo[Number].PlayerID = Other.PlayerReplicationInfo.PlayerID;
}

//Gets Players IP and returns it as a string
function string FilterAddress(Pawn Other)
{
	local int i, j;
	local string TempAddress;

	TempAddress = PlayerPawn(Other).GetPlayerNetworkAddress();

	j = InStr(TempAddress,":");
	if( j != -1 )
		TempAddress = Left(TempAddress,j);

	return TempAddress;
}

function Mutate(string MutateString, PlayerPawn Sender)
{
	local bool bAdmin;
	local int i;

	bAdmin = Sender.PlayerReplicationInfo.bAdmin;

	if ( NextMutator != None )
		NextMutator.Mutate(MutateString, Sender);

	if ( ( Left( Caps (MutateString),9) == "PERMAMUTE" ) )
	{
		for (i=0; i<ArrayCount(PInfo);i++)
		{
			if ( Sender.PlayerReplicationInfo.PlayerID == PInfo[i].PlayerID )
			{
				if ( Mid(MutateString,10) == "?" )
				{
					if ( PInfo[i].SM.LocalPerma )
						PInfo[i].Player.ClientMessage("PERMAMUTE IS ON!");
					else
						PInfo[i].Player.ClientMessage("PERMAMUTE IS OFF!");
				}
				else
				{
					PInfo[i].SM.SetPermaMute(!PInfo[i].SM.LocalPerma);
					break;
				}
			}
		}
	}
	else if ( ( Left( Caps (MutateString),7) == "MAPMUTE" ) )
	{
		for (i=0; i<ArrayCount(PInfo);i++)
		{
			if ( Sender.PlayerReplicationInfo.PlayerID == PInfo[i].PlayerID )
			{
				if ( Mid(MutateString,8) == "?" )
				{
					if ( PInfo[i].SM.LocalMapM )
						PInfo[i].Player.ClientMessage("MAPMUTE IS ON!");
					else
						PInfo[i].Player.ClientMessage("MAPMUTE IS OFF!");
				}
				else
				{
					PInfo[i].SM.SetMapMute(!PInfo[i].SM.LocalMapM);
					break;
				}
			}
		}
	}
	else if ( bAdmin )
	{
		if ( ( Left( Caps (MutateString),10) == "MUTEPLAYER" ) )
		{
			if ( int(Mid(MutateString,11,1)) == 0 )
			{
				for (i=0; i<ArrayCount(PInfo) && PInfo[i].Player != none;i++)
				{
					if ( Mid(MutateString,12) ~= caps(PInfo[i].PlayerName ) )
					{
						PInfo[i].SM.SetDebug(1);
						PInfo[i].SM.GiveServerDebug(1);
						break;
					}
				}
			}
			else
			{
				for (i=0; i<ArrayCount(PInfo) && PInfo[i].Player != none;i++)
				{
					if ( int(Mid(MutateString,12)) == PInfo[i].PlayerID )
					{
						PInfo[i].SM.SetDebug(1);
						PInfo[i].SM.GiveServerDebug(1);
						break;
					}
				}
			}
		}
		else if ( ( Left( Caps (MutateString),12) == "UNMUTEPLAYER" ) )
		{
			if ( int(Mid(MutateString,13,1)) == 0 )
			{
				for (i=0; i<ArrayCount(PInfo) && PInfo[i].Player != none;i++)
				{
					if ( Mid(MutateString,14) ~= caps(PInfo[i].PlayerName ) )
					{
						PInfo[i].SM.SetDebug(0);
						PInfo[i].SM.GiveServerDebug(0);
						break;
					}
				}
			}
			else
			{
				for (i=0; i<ArrayCount(PInfo) && PInfo[i].Player != none;i++)
				{
					if ( int(Mid(MutateString,14)) == PInfo[i].PlayerID )
					{
						PInfo[i].SM.SetDebug(0);
						PInfo[i].SM.GiveServerDebug(0);
						break;
					}
				}
			}
		}
	}
}

function bool MutatorTeamMessage(Actor Sender, Pawn Receiver, PlayerReplicationInfo PRI, coerce string S, name Type, optional bool bBeep)
{
	local int i;

	if (PRI != None && Receiver != None && Sender.IsA('PlayerPawn') && !PlayerPawn(Sender).PlayerReplicationInfo.bIsSpectator )
	{
		if ( Left( caps(S), 10) ~= "!PERMAMUTE" )
		{
			if ( Sender == Receiver )
			{
				if ( Mid(S,11) == "?" )
					PlayerPawn(Sender).ConsoleCommand("MUTATE PERMAMUTE ?");
				else
					PlayerPawn(Sender).ConsoleCommand("MUTATE PERMAMUTE");
			}
		}
		else	if ( Left( caps(S), 8) ~= "!MAPMUTE" )
		{
			if ( Sender == Receiver )
			{
				if ( Mid(S,9) == "?" )
					PlayerPawn(Sender).ConsoleCommand("MUTATE MAPMUTE ?");
				else
					PlayerPawn(Sender).ConsoleCommand("MUTATE MAPMUTE");
			}
		}
		else if ( Left( caps(S), 9) ~= "!MUTENAME" )
		{
			if ( Sender == Receiver )
			{
				S = Mid(S,10);
				PlayerPawn(Sender).ConsoleCommand("MUTATE MUTEPLAYER 0"$caps(S));
			}
		}
		else if ( Left( caps(S), 11) ~= "!UNMUTENAME" )
		{
			if ( Sender == Receiver )
			{
				S = Mid(S,12);
				PlayerPawn(Sender).ConsoleCommand("MUTATE UNMUTEPLAYER 0"$caps(S));
			}
		}
		else if ( Left( caps(S), 7) ~= "!MUTEID" )
		{
			if ( Sender == Receiver )
			{
				S = Mid(S,8);
				PlayerPawn(Sender).ConsoleCommand("MUTATE MUTEPLAYER 1"$caps(S));
			}
		}
		else if ( Left( caps(S), 9) ~= "!UNMUTEID" )
		{
			if ( Sender == Receiver )
			{
				S = Mid(S,10);
				PlayerPawn(Sender).ConsoleCommand("MUTATE UNMUTEPLAYER 1"$caps(S));
			}
		}
		else if ( Left( caps(S), 9) ~= "!MUTELIST" && PlayerPawn(Sender).PlayerReplicationInfo.bAdmin)
		{
			if ( Sender == Receiver )
			{
				PlayerPawn(Sender).ClientMessage("## Loco_Chat Banned Chatters List");
				PlayerPawn(Sender).ClientMessage("---------------------------------");

				for(i=0;i<ArrayCount(PInfo) && PInfo[i].Player != none && Sender == Receiver;i++)
				{
					if ( PInfo[i].SM.LocalDebug == 1 )
						PlayerPawn(Sender).ClientMessage(" Name: "$PInfo[i].PlayerName$"     ID: "$PInfo[i].PlayerID);
				}
				PlayerPawn(Sender).ClientMessage("---------------------------------");
			}
		}
		else
		{
			for(i=0;i<ArrayCount(PInfo) && PInfo[i].Player != none && Sender == Receiver;i++)
			{
				if ( PInfo[i].PlayerID == -1 )
					break;
				if ( Pawn(Sender).PlayerReplicationInfo.PlayerID == PInfo[i].PlayerID && PInfo[i].SM.LocalDebug != 1  )
				{
					for(i=0;i<ArrayCount(ChatWords) && ChatWords[i] != "";i++)
					{
						if ( S ~= ChatWords[i] )
						{
							PlayMedia( 1, i );

							if ( NextMessageMutator != None )
								return NextMessageMutator.MutatorTeamMessage( Sender, Receiver, PRI, S, Type, bBeep );

							return true;
						}
					}
					if ( bAllowStatic )
					{
						for(i=0;i<ArrayCount(StaticWords) && StaticWords[i] != "";i++)
						{
							if ( Instr(caps(S), caps(StaticWords[i]) ) >= 0 )
							{
								PlayMedia( 0, i );

								if ( NextMessageMutator != None )
									return NextMessageMutator.MutatorTeamMessage( Sender, Receiver, PRI, S, Type, bBeep );

								return true;
							}
						}
					}
				}
			}
		}
	}

	if ( NextMessageMutator != None )
		return NextMessageMutator.MutatorTeamMessage( Sender, Receiver, PRI, S, Type, bBeep );

	return true;
}

//Type 0 is for Static Chat Words i.e., the word "bye" will play sounds when "goodbye", "buhbye", etc. is said
//Type 1 is for StandAlone words i.e., "LOL" and not "OH LOL"
function PlayMedia(int Type, int pSound )
{
	local int i;

	if ( Level.TimeSeconds >= PlayNext )
	{
		for (i=0;i<ArrayCount(PInfo) && PInfo[i].Player != none; i++)
		{
			if( !PInfo[i].SM.LocalPerma && !PInfo[i].SM.LocalMapM )
		 	{
				switch (Type)
				{
					case 0:
						PlayerPawn(PInfo[i].Player).ClientPlaySound(sndSChat[pSound],false,true);
						PlayNext = Level.TimeSeconds + StaticDuration[pSound];
						break;
					case 1:
						PlayerPawn(PInfo[i].Player).ClientPlaySound(sndChat[pSound],false,true);
						PlayNext = Level.TimeSeconds + ChatDuration[pSound];
						break;
				}
			}
		}
	}
}

defaultproperties
{
}
